@@ -17,13 +17,13 @@ typedef struct {
 } FILE_INFO;
 
 
-BOOL cflash_init();
+BOOL cflash_init( const char *disk_image_filename);
 
 unsigned int cflash_read(unsigned int address);
 
 void cflash_write(unsigned int address,unsigned int data);
 
-void cflash_close();
+void cflash_close( void);
 
 #ifdef __cplusplus
 }
